@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2016	PgPool Global Development Group
+ * Copyright (c) 2003-2011	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -58,6 +58,7 @@
 #include "pool_process_reporting.h"
 
 #define MAX_FILE_LINE_LEN    512
+#define MAX_USER_PASSWD_LEN  128
 
 extern void pcp_set_timeout(long sec);
 volatile sig_atomic_t pcp_exit_request; /* non 0 means SIGTERM(smart shutdown) or SIGINT(fast shutdown) has arrived */
@@ -177,13 +178,6 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 		}
 
 		rsize = ntohl(rsize);
-
-		if (rsize <= 0 || rsize >= MAX_PCP_PACKET_LENGTH)
-		{
-			pool_error("pcp_child: incorrect packet length (%d)", rsize);
-			exit(1);
-		}
-
 		if ((rsize - sizeof(int)) > 0)
 		{
 			buf = (char *)malloc(rsize - sizeof(int));
@@ -419,7 +413,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 
 				process_list = pool_get_process_list(&process_count);
 
-                mesg = (char *)malloc(7*process_count); /* PID is at most 6 characters long */
+				mesg = (char *)malloc(6*process_count);	/* port# is at most 5 characters long (MAX:65535) */
 				if (mesg == NULL)
 				{
 					pool_error("pcp_child: malloc() failed. reason: %s", strerror(errno));
@@ -430,10 +424,10 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 
 				for (i = 0; i < process_count; i++)
 				{
-                    char process_id[7];
-                    snprintf(process_id, sizeof(process_id), "%d", process_list[i]);
-                    snprintf(mesg+total_port_len, strlen(process_id)+1, "%s", process_id);
-                    total_port_len += strlen(process_id)+1;
+					char port[6];
+					snprintf(port, sizeof(port), "%d", process_list[i]);
+					snprintf(mesg+total_port_len, strlen(port)+1, "%s", port);
+					total_port_len += strlen(port)+1;
 				}
 
 				pcp_write(frontend, "n", 1);
@@ -758,7 +752,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 			{
 				int node_id;
 				int wsize;
-				char *code;
+				char code[] = "CommandComplete";
 				bool gracefully;
 
 				if (tos == 'D')
@@ -768,15 +762,12 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 
 				node_id = atoi(buf);
 				pool_debug("pcp_child: detaching Node ID %d", node_id);
-				if (pool_detach_node(node_id, gracefully) == 0)
-					code = "CommandComplete";
-				else
-					code = "CommandFailed";
+				pool_detach_node(node_id, gracefully);
 
 				pcp_write(frontend, "d", 1);
-				wsize = htonl(strlen(code) + 1 + sizeof(int));
+				wsize = htonl(sizeof(code) + sizeof(int));
 				pcp_write(frontend, &wsize, sizeof(int));
-				pcp_write(frontend, code, strlen(code) + 1);
+				pcp_write(frontend, code, sizeof(code));
 				if (pcp_flush(frontend) < 0)
 				{
 					pool_error("pcp_child: pcp_flush() failed. reason: %s", strerror(errno));
@@ -873,7 +864,7 @@ pcp_do_child(int unix_fd, int inet_fd, char *pcp_conf_file)
 					if (MASTER_SLAVE && !strcmp(pool_config->master_slave_sub_mode, MODE_STREAMREP))
 						msg = "primary server cannot be recovered by online recovery.";
 					else
-						msg = "recovery request is only allowed in replication and streaming replication modes.";
+						msg = "recovery request is accepted only in replication mode or stereaming replication mode. ";
 
 					len = strlen(msg)+1;
 					pcp_write(frontend, "e", 1);
@@ -1242,13 +1233,13 @@ user_authenticate(char *buf, char *passwd_file, char *salt, int salt_len)
 
 	/* strcpy() should be OK, but use strncpy() to be extra careful */
 	strncpy(packet_username, buf, MAX_USER_PASSWD_LEN);
-	index = (char *) memchr(buf, '\0', MAX_USER_PASSWD_LEN);
+	index = (char *) memchr(buf, '\0', MAX_USER_PASSWD_LEN) + 1;
 	if (index == NULL)
 	{
 		pool_debug("pcp_child: error while reading authentication packet");
 		return 0;
 	}
-	strncpy(packet_password, ++index, MAX_USER_PASSWD_LEN);
+	strncpy(packet_password, index, MAX_USER_PASSWD_LEN);
 
 	fp = fopen(passwd_file, "r");
 	if (fp == NULL)
@@ -1328,30 +1319,16 @@ static RETSIGTYPE reload_config_handler(int sig)
 /* Dedatch a node */
 static int pool_detach_node(int node_id, bool gracefully)
 {
-	int nRet = 0;
 	if (!gracefully)
 	{
-		if (degenerate_backend_set_ex(&node_id, 1, false) == false)
-		{
-			pool_error("pcp_child: processing detach node failed");
-			return -1;
-		}
+		notice_backend_error(node_id);	/* send failover request */
 		return 0;
 	}
-
-	/* Check if the NODE DOWN can be executed on
-	 * the given node id.
-	 */
-	if (degenerate_backend_set_ex(&node_id, 1, true) == false)
-	{
-		pool_error("pcp_child: processing graceful detach node failed");
-		return -1;
-	}
-
+		
 	/*
 	 * Wait until all frontends exit
 	 */
-	*InRecovery = 1;	/* This wiil ensure that new incoming
+	*InRecovery = RECOVERY_DETACH;	/* This wiil ensure that new incoming
 						 * connection requests are blocked */
 
 	if (wait_connection_closed())
@@ -1364,19 +1341,12 @@ static int pool_detach_node(int node_id, bool gracefully)
 	/*
 	 * Now all frontends have gone. Let's do failover.
 	 */
-	if (degenerate_backend_set_ex(&node_id, 1, false) == false)
-	{
-		nRet = -1;
-		pcp_wakeup_request = 1;
-		pool_error("pcp_child: processing graceful detach node failed");
-	}
-	else
-	{
-		/*
-		 * Wait for failover completed.
-		 */
-		pcp_wakeup_request = 0;
-	}
+	notice_backend_error(node_id);		/* send failover request */
+
+	/*
+	 * Wait for failover completed.
+	 */
+	pcp_wakeup_request = 0;
 
 	while (!pcp_wakeup_request)
 	{
@@ -1391,7 +1361,7 @@ static int pool_detach_node(int node_id, bool gracefully)
 	 */
 	finish_recovery();
 
-	return nRet;
+	return 0;
 }
 
 /* Promote a node */
@@ -1402,11 +1372,11 @@ static int pool_promote_node(int node_id, bool gracefully)
 		promote_backend(node_id);	/* send promote request */
 		return 0;
 	}
-
+		
 	/*
 	 * Wait until all frontends exit
 	 */
-	*InRecovery = 1;	/* This wiil ensure that new incoming
+	*InRecovery = RECOVERY_PROMOTE;	/* This wiil ensure that new incoming
 						 * connection requests are blocked */
 
 	if (wait_connection_closed())
