@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2013	PgPool Global Development Group
+ * Copyright (c) 2003-2012	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -73,13 +73,7 @@ void pool_ssl_negotiate_clientserver(POOL_CONNECTION *cp) {
 
 	pool_debug("pool_ssl: sending client->server SSL request");
 	pool_write_and_flush(cp, ssl_packet, sizeof(int)*2);
-
-	if (pool_read(cp, &server_response, 1) < 0)
-	{
-		pool_error("pool_ssl_negotiate_clientserver: pool_read failed");
-		return;
-	}
-
+	pool_read(cp, &server_response, 1);
 	pool_debug("pool_ssl: client->server SSL response: %c", server_response);
 
 	switch (server_response) {
@@ -173,16 +167,64 @@ int pool_ssl_read(POOL_CONNECTION *cp, void *buf, int size) {
 			n = -1;
 			break;
 		default:
-			perror_ssl("SSL_read");
-			n = -1;
+			pool_error("pool_ssl_read: unrecognized error code: %d", err);
+			/*
+			 * We assume that the connection is broken. Returns 0
+			 * rather than -1 in this case because -1 triggers
+			 * unwanted failover in the caller (pool_read).
+			 */
+			n = 0;
 			break;
 	}
 
 	return n;
 }
 
-int pool_ssl_write(POOL_CONNECTION *cp, const void *buf, int size) {
-	return SSL_write(cp->ssl, buf, size);
+int pool_ssl_write(POOL_CONNECTION *cp, const void *buf, int size)
+{
+	int n;
+	int err;
+
+retry:
+	errno = 0;
+	n = SSL_write(cp->ssl, buf, size);
+	err = SSL_get_error(cp->ssl, n);
+	switch (err)
+	{
+		case SSL_ERROR_NONE:
+			break;
+
+		case SSL_ERROR_WANT_READ:
+		case SSL_ERROR_WANT_WRITE:
+			goto retry;
+
+		case SSL_ERROR_SYSCALL:
+			if (n == -1)
+			{
+				pool_error("SSL_write error: %d", err);
+			}
+			else
+			{
+				pool_error("SSL_write error: EOF detected");
+				n = -1;
+			}
+			break;
+
+		case SSL_ERROR_SSL:
+		case SSL_ERROR_ZERO_RETURN:
+			perror_ssl("SSL_write");
+			n = -1;
+			break;
+
+		default:
+			pool_error("pool_ssl_write: unrecognized error code: %d", err);
+			/*
+			 * We assume that the connection is broken.
+			 */
+			n = -1;
+			break;
+	}
+	return n;
 }
 
 static int init_ssl_ctx(POOL_CONNECTION *cp, enum ssl_conn_type conntype) {
@@ -190,28 +232,19 @@ static int init_ssl_ctx(POOL_CONNECTION *cp, enum ssl_conn_type conntype) {
 	char *cacert = NULL, *cacert_dir = NULL;
 
 	/* initialize SSL members */
-#if (OPENSSL_VERSION_NUMBER >= 0x10100000L)
-		cp->ssl_ctx = SSL_CTX_new(TLS_method());
-#else
-		cp->ssl_ctx = SSL_CTX_new(SSLv23_method());
-#endif
-
+	cp->ssl_ctx = SSL_CTX_new(TLSv1_method());
 	SSL_RETURN_ERROR_IF( (! cp->ssl_ctx), "SSL_CTX_new" );
-	/*
-	 * Disable OpenSSL's moving-write-buffer sanity check, because it
-	 * causes unnecessary failures in nonblocking send cases.
-	 */
-	SSL_CTX_set_mode(cp->ssl_ctx, SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
 
 	if ( conntype == ssl_conn_serverclient) {
-		error = SSL_CTX_use_certificate_chain_file(cp->ssl_ctx,
-		                                     pool_config->ssl_cert);
-		SSL_RETURN_ERROR_IF( (error != 1), "Loading SSL certificate");
+		error = SSL_CTX_use_certificate_file(cp->ssl_ctx,
+		                                     pool_config->ssl_cert,
+		                                     SSL_FILETYPE_PEM);
+		SSL_RETURN_ERROR_IF( (error <= 0), "Loading SSL certificate");
 
 		error = SSL_CTX_use_PrivateKey_file(cp->ssl_ctx,
 		                                    pool_config->ssl_key,
 		                                    SSL_FILETYPE_PEM);
-		SSL_RETURN_ERROR_IF( (error != 1), "Loading SSL private key");
+		SSL_RETURN_ERROR_IF( (error <= 0), "Loading SSL private key");
 	} else {
 		/* set extra verification if ssl_ca_cert or ssl_ca_cert_dir are set */
 		if (strlen(pool_config->ssl_ca_cert))
@@ -220,10 +253,10 @@ static int init_ssl_ctx(POOL_CONNECTION *cp, enum ssl_conn_type conntype) {
 			cacert_dir = pool_config->ssl_ca_cert_dir;
     
 		if ( cacert || cacert_dir ) {
-			error = SSL_CTX_load_verify_locations(cp->ssl_ctx,
+			error = (!SSL_CTX_load_verify_locations(cp->ssl_ctx,
 			                                        cacert,
-			                                        cacert_dir);
-			SSL_RETURN_ERROR_IF((error != 1), "SSL verification setup");
+			                                        cacert_dir));
+			SSL_RETURN_ERROR_IF(error, "SSL verification setup");
 			SSL_CTX_set_verify(cp->ssl_ctx, SSL_VERIFY_PEER, NULL);
 		}
 	}
