@@ -5,7 +5,7 @@
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2016	PgPool Global Development Group
+ * Copyright (c) 2003-2011	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -32,6 +32,7 @@
 #include "parser/parser.h"
 #include "parser/pool_memory.h"
 
+
 typedef struct {
 	char	*attrname;	/* attribute name */
 	char	*adsrc;		/* default value expression */
@@ -47,17 +48,16 @@ typedef struct {
 	A_Const					*ts_const;
 	POOL_CONNECTION_POOL	*backend;
 	char					*relname;
-	int						 num_params;		/* num of original params (for Parse) */
-	bool		 			 rewrite_to_params;	/* true if rewritten to param insread of const */
-	bool		 			 rewrite;			/* has rewritten? */
-	List					*params;			/* list of additional params */
+	int						 num_params;	/* num of original params (for Parse) */
+	bool		 			 rewrite_to_params;
+	bool		 			 rewrite;		/* has rewritten? */
+	List					*params;		/* list of additional params */
 } TSRewriteContext;
 
 static void *ts_register_func(POOL_SELECT_RESULT *res);
 static void *ts_unregister_func(void *data);
 static TSRel *relcache_lookup(TSRewriteContext *ctx);
 static bool isStringConst(Node *node, const char *str);
-static bool isSystemType(Node *node, const char *name);
 static bool rewrite_timestamp_walker(Node *node, void *context);
 static bool rewrite_timestamp_insert(InsertStmt *i_stmt, TSRewriteContext *ctx);
 static bool rewrite_timestamp_update(UpdateStmt *u_stmt, TSRewriteContext *ctx);
@@ -66,7 +66,6 @@ static Node *makeTsExpr(TSRewriteContext *ctx);
 static A_Const *makeStringConstFromQuery(POOL_CONNECTION_POOL *backend, char *expression);
 bool raw_expression_tree_walker(Node *node, bool (*walker) (), void *context);
 
-#define		MAX_RELCACHE 32
 POOL_RELCACHE	*ts_relcache;
 
 
@@ -112,10 +111,13 @@ ts_register_func(POOL_SELECT_RESULT *res)
 static void *
 ts_unregister_func(void *data)
 {
-    TSRel   *rel = (TSRel *) data;
-    if (rel)
-        free(rel);
-    return (void *)0;
+	TSRel	*rel = (TSRel *) data;
+
+	if (rel == NULL)
+		return NULL;
+
+	free(rel);
+	return rel;
 }
 
 
@@ -159,7 +161,7 @@ relcache_lookup(TSRewriteContext *ctx)
 
 	if (!ts_relcache)
 	{
-		ts_relcache = pool_create_relcache(MAX_RELCACHE, query, ts_register_func, ts_unregister_func, false);
+		ts_relcache = pool_create_relcache(pool_config->relcache_size, query, ts_register_func, ts_unregister_func, false);
 
 		if (ts_relcache == NULL)
 		{
@@ -171,6 +173,7 @@ relcache_lookup(TSRewriteContext *ctx)
 	return (TSRel *) pool_search_relcache(ts_relcache, ctx->backend, ctx->relname);
 }
 
+
 static Node *
 makeTsExpr(TSRewriteContext *ctx)
 {
@@ -205,7 +208,7 @@ isStringConst(Node *node, const char *str)
 }
 
 
-static bool
+bool
 isSystemType(Node *node, const char *name)
 {
 	TypeName	*typename;
@@ -309,13 +312,12 @@ get_current_timestamp(POOL_CONNECTION_POOL *backend)
 {
 	POOL_SELECT_RESULT *res;
 	POOL_STATUS		 status;
-	static char		timestamp[64];
+	static char		timestamp[32];
 
 	status = do_query(MASTER(backend), "SELECT now()", &res, MAJOR(backend));
 	if (status != POOL_CONTINUE)
 	{
-		pool_error("get_current_timestamp: do_query failed");
-		free_select_result(res);
+		pool_error("get_current_timestamp: do_query faild");
 		return NULL;
 	}
 
@@ -684,10 +686,6 @@ rewrite_timestamp(POOL_CONNECTION_POOL *backend, Node *node,
 	else
 		;
 
-	/* save number of parameters in original query */
-	if (message)
-		message->query_context->num_original_params = ctx.num_params;
-
 	if (!rewrite)
 		return NULL;
 
@@ -748,8 +746,7 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 	int32		 tmp4;
 	int			 i,
 				 ts_len,
-				 copy_len,
-				 num_org_params;
+				 copy_len;
 	const char	*copy_from;
 	char		*ts,
 				*copy_to,
@@ -772,11 +769,8 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 
 	ts_len = strlen(ts);
 
-	/* enlarge length for timestamp parameters */
-	*len += (ts_len + sizeof(int32)) * message->num_tsparams;
-	/* allocate extra memory for parameter formats */
-	num_org_params = message->query_context->num_original_params;
-	new_msg = copy_to = (char *) malloc(*len + sizeof(int16) * (message->num_tsparams + num_org_params));
+	*len += (strlen(ts) + sizeof(int32)) * message->num_tsparams;
+	new_msg = copy_to = (char *) malloc(*len + message->num_tsparams * sizeof(int16));
 	copy_from = orig_msg;
 
 	/* portal_name */
@@ -794,20 +788,12 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 	copy_len = sizeof(int16);
 	tmp2 = num_formats = ntohs(tmp2);
 
-	if (num_formats >= 1)
+	if (num_formats > 1)
 	{
-		/* one means the specified format code is applied all original parameters */
-		if (num_formats == 1)
-		{
-			*len += (num_org_params - 1) * sizeof(int16);
-			tmp2 += num_org_params - 1;
-		}
-
-		/* enlarge message length for timestamp parameter's formats */
+		/* enlarge message length */
 		*len += message->num_tsparams * sizeof(int16);
 		tmp2 += message->num_tsparams;
 	}
-
 	tmp2 = htons(tmp2);
 	memcpy(copy_to, &tmp2, copy_len);	/* copy number of format codes */
 	copy_to += copy_len; copy_from += copy_len;
@@ -817,17 +803,10 @@ bind_rewrite_timestamp(POOL_CONNECTION_POOL *backend,
 	memcpy(copy_to, copy_from, copy_len);		/* copy format codes */
 	copy_to += copy_len; copy_from += copy_len;
 
-	if (num_formats >= 1)
+	if (num_formats > 1)
 	{
-		/* copy the specified format code as numbers of original parameters */
-		if (num_formats == 1)
-		{
-			memcpy(copy_to, copy_from, (num_org_params - 1) * sizeof(int16));
-			copy_to += (num_org_params - 1) * sizeof(int16);
-		}
-
-		/* set additional format codes to zero(text) */
-		memset(copy_to, 0, message->num_tsparams * sizeof(int16));
+		/* set format codes to zero(text) */
+		memset(copy_to, 0, message->num_tsparams * 2);
 		copy_to += sizeof(int16) * message->num_tsparams;
 	}
 
@@ -889,8 +868,7 @@ static A_Const *makeStringConstFromQuery(POOL_CONNECTION_POOL *backend, char *ex
 	status = do_query(MASTER(backend), query, &res, MAJOR(backend));
 	if (status != POOL_CONTINUE)
 	{
-		pool_error("makeStringConstFromQuery: do_query failed");
-		free_select_result(res);
+		pool_error("makeStringConstFromQuery: do_query faild");
 		return NULL;
 	}
 
